NETWORK_ERROR_RETRY_COUNT = 50
OAUTH_LOCAL_PORT = 10500
OAUTH_HOST = "localhost"
SYNC_CHUNK_MAX_RESULTS = 200
SYNC_MAX_DOWNLOAD_WORKERS = 5
SYNC_DOWNLOAD_CACHE_MEMORY_LIMIT = 256
DATABASE_NAME = "en_backup.db"
BACKEND = "evernote"

SYNC_CHUNK_MAX_RESULTS_SERVER_LIMIT = 256
SYNC_MAX_DOWNLOAD_WORKERS_SANE_LIMIT = 256
